@@ -4,6 +4,7 @@
 #include "v7/driver.h"
 #include "dballe/sql/sql.h"
 #include "dballe/sql/sqlite.h"
+#include "dballe/db/summary_memory.h"
 #include <wreport/error.h>
 #include <algorithm>
 #include <unistd.h>
@@ -22,7 +23,7 @@ impl::Messages messages_from_db(std::shared_ptr<db::Transaction> tr, const dball
     impl::Messages res;
     auto cursor = tr->query_messages(query);
     while (cursor->next())
-        res.emplace_back(cursor->detach_message());
+        res.emplace_back(cursor->get_message());
     return res;
 }
 
@@ -31,19 +32,19 @@ impl::Messages messages_from_db(std::shared_ptr<db::Transaction> tr, const char*
     impl::Messages res;
     auto cursor = tr->query_messages(*dballe::tests::query_from_string(query));
     while (cursor->next())
-        res.emplace_back(cursor->detach_message());
+        res.emplace_back(cursor->get_message());
     return res;
 }
 
 
 void ActualCursor::station_keys_match(const DBStation& expected)
 {
-    wassert(actual(_actual.get_station()) == expected);
+    wassert(actual(_actual->get_station()) == expected);
 }
 
 void ActualCursor::data_context_matches(const Data& expected)
 {
-    db::CursorData* c = dynamic_cast<db::CursorData*>(&_actual);
+    auto c = std::dynamic_pointer_cast<db::CursorData>(_actual);
     if (!c) throw TestFailed("cursor is not an instance of CursorData");
 
     const core::Data& exp = core::Data::downcast(expected);
@@ -60,12 +61,12 @@ void ActualCursor::data_context_matches(const Data& expected)
 
 void ActualCursor::data_var_matches(const wreport::Var& expected)
 {
-    if (db::CursorStationData* c = dynamic_cast<db::CursorStationData*>(&_actual))
+    if (auto c = std::dynamic_pointer_cast<db::CursorStationData>(_actual))
     {
         wassert(actual(c->get_varcode()) == expected.code());
         wassert(actual(c->get_var()) == expected);
     }
-    else if (db::CursorData* c = dynamic_cast<db::CursorData*>(&_actual))
+    else if (auto c = std::dynamic_pointer_cast<db::CursorData>(_actual))
     {
         wassert(actual(c->get_varcode()) == expected.code());
         wassert(actual(c->get_var()) == expected);
@@ -92,7 +93,7 @@ template<typename DB>
 void ActualDB<DB>::try_data_query(const Query& query, unsigned expected)
 {
     // Run the query
-    unique_ptr<Cursor> cur = this->_actual->query_data(query);
+    auto cur = this->_actual->query_data(query);
 
     // Check the number of results
     wassert(actual(cur->remaining()) == expected);
@@ -104,7 +105,7 @@ template<typename DB>
 void ActualDB<DB>::try_station_query(const std::string& query, unsigned expected)
 {
     // Run the query
-    unique_ptr<Cursor> cur = this->_actual->query_stations(core_query_from_string(query));
+    auto cur = this->_actual->query_stations(core_query_from_string(query));
 
     // Check the number of results
     wassert(actual(cur->remaining()) == expected);
@@ -123,7 +124,7 @@ void ActualDB<DB>::try_summary_query(const std::string& query, unsigned expected
     if (cur->remaining() != 0)
         wassert(actual(cur->remaining()) == expected);
 
-    db::DBSummary summary;
+    db::DBSummaryMemory summary;
     unsigned found = 0;
     while (cur->next())
     {
@@ -186,6 +187,18 @@ void BaseDBFixture<DB>::test_setup()
 
     if (!db)
         create_db();
+
+    destroys_db = false;
+}
+
+template<typename DB>
+void BaseDBFixture<DB>::test_teardown()
+{
+    if (destroys_db)
+    {
+        db->disappear();
+        db.reset();
+    }
 }
 
 template<typename DB>
@@ -234,7 +247,7 @@ std::shared_ptr<dballe::db::v7::DB> V7DB::create_db(const std::string& backend,
 }
 
 
-void TestDataSet::populate_db(DB& db)
+void TestDataSet::populate_db(dballe::DB& db)
 {
     auto tr = db.transaction();
     populate_transaction(*tr);
