@@ -16,8 +16,7 @@ class Tests : public TestCase
     void register_tests() override;
 } test("dballe_importer");
 
-void Tests::register_tests()
-{
+void Tests::register_tests() {
 
 add_method("options", []() {
     impl::ImporterOptions simplified;
@@ -27,6 +26,9 @@ add_method("options", []() {
     wassert(actual(impl::ImporterOptions("") == simplified).istrue());
     wassert(actual(impl::ImporterOptions("simplified") == simplified).istrue());
     wassert(actual(impl::ImporterOptions("accurate") == accurate).istrue());
+
+    wassert_true(simplified.domain_errors == ImporterOptions::DomainErrors::THROW);
+    wassert_true(accurate.domain_errors == ImporterOptions::DomainErrors::THROW);
 });
 
 }
