@@ -1,4 +1,5 @@
 #include "wr_codec.h"
+#include "domain_errors.h"
 #include "dballe/file.h"
 #include "msg.h"
 #include "context.h"
@@ -6,6 +7,7 @@
 #include "wr_importers/base.h"
 #include <wreport/bulletin.h>
 #include <wreport/vartable.h>
+#include <wreport/options.h>
 
 using namespace wreport;
 using namespace std;
@@ -15,13 +17,13 @@ namespace impl {
 namespace msg {
 
 WRImporter::WRImporter(const dballe::ImporterOptions& opts)
-    : Importer(opts) {}
+    : BulletinImporter(opts) {}
 
 BufrImporter::BufrImporter(const dballe::ImporterOptions& opts)
     : WRImporter(opts) {}
 BufrImporter::~BufrImporter() {}
 
-bool BufrImporter::foreach_decoded(const BinaryMessage& msg, std::function<bool(std::unique_ptr<dballe::Message>)> dest) const
+bool BufrImporter::foreach_decoded(const BinaryMessage& msg, std::function<bool(std::shared_ptr<dballe::Message>)> dest) const
 {
     unique_ptr<BufrBulletin> bulletin(BufrBulletin::decode(msg.data));
     return foreach_decoded_bulletin(*bulletin, dest);
@@ -31,7 +33,7 @@ CrexImporter::CrexImporter(const dballe::ImporterOptions& opts)
     : WRImporter(opts) {}
 CrexImporter::~CrexImporter() {}
 
-bool CrexImporter::foreach_decoded(const BinaryMessage& msg, std::function<bool(std::unique_ptr<dballe::Message>)> dest) const
+bool CrexImporter::foreach_decoded(const BinaryMessage& msg, std::function<bool(std::shared_ptr<dballe::Message>)> dest) const
 {
     unique_ptr<CrexBulletin> bulletin(CrexBulletin::decode(msg.data));
     return foreach_decoded_bulletin(*bulletin, dest);
@@ -40,12 +42,14 @@ bool CrexImporter::foreach_decoded(const BinaryMessage& msg, std::function<bool(
 Messages WRImporter::from_bulletin(const wreport::Bulletin& msg) const
 {
     Messages res;
-    foreach_decoded_bulletin(msg, [&](unique_ptr<dballe::Message>&& m) { res.emplace_back(move(m)); return true; });
+    foreach_decoded_bulletin(msg, [&](std::shared_ptr<dballe::Message>&& m) { res.emplace_back(move(m)); return true; });
     return res;
 }
 
-bool WRImporter::foreach_decoded_bulletin(const wreport::Bulletin& msg, std::function<bool(std::unique_ptr<dballe::Message>)> dest) const
+bool WRImporter::foreach_decoded_bulletin(const wreport::Bulletin& msg, std::function<bool(std::shared_ptr<dballe::Message>)> dest) const
 {
+    WreportVarOptionsForImport wreport_config(opts.domain_errors);
+
     // Infer the right importer. See Common Code Table C-13
     std::unique_ptr<wr::Importer> importer;
     switch (msg.data_category)
@@ -91,10 +95,10 @@ bool WRImporter::foreach_decoded_bulletin(const wreport::Bulletin& msg, std::fun
     MessageType type = importer->scanType(msg);
     for (unsigned i = 0; i < msg.subsets.size(); ++i)
     {
-        std::unique_ptr<Message> newmsg(new Message);
+        auto newmsg = std::make_shared<Message>();
         newmsg->type = type;
         importer->import(msg.subsets[i], *newmsg);
-        if (!dest(move(newmsg)))
+        if (!dest(newmsg))
             return false;
     }
     return true;
@@ -102,7 +106,7 @@ bool WRImporter::foreach_decoded_bulletin(const wreport::Bulletin& msg, std::fun
 
 
 WRExporter::WRExporter(const dballe::ExporterOptions& opts)
-    : Exporter(opts) {}
+    : BulletinExporter(opts) {}
 
 BufrExporter::BufrExporter(const dballe::ExporterOptions& opts)
     : WRExporter(opts) {}
