@@ -59,7 +59,7 @@ struct Row
 
     operator bool() const { return row != nullptr; }
     operator MYSQL_ROW() { return row; }
-    operator const MYSQL_ROW() const { return row; }
+    operator MYSQL_ROW() const { return row; }
 
     int as_int(unsigned col) const { return strtol(row[col], 0, 10); }
     unsigned as_unsigned(unsigned col) const { return strtoul(row[col], 0, 10); }
@@ -128,6 +128,8 @@ class MySQLConnection : public Connection
 protected:
     /// Database connection
     MYSQL* db = nullptr;
+    /// Marker to catch attempts to reuse connections in forked processes
+    bool forked = false;
 
     void send_result(mysql::Result&& res, std::function<void(const mysql::Row&)> dest);
 
@@ -137,14 +139,23 @@ protected:
     // See https://dev.mysql.com/doc/refman/5.0/en/mysql-real-connect.html
     void open(const mysql::ConnectInfo& info);
 
-public:
     MySQLConnection();
+
+    void fork_prepare() override;
+    void fork_parent() override;
+    void fork_child() override;
+
+    void check_connection();
+
+public:
     MySQLConnection(const MySQLConnection&) = delete;
     MySQLConnection(const MySQLConnection&&) = delete;
     ~MySQLConnection();
     MySQLConnection& operator=(const MySQLConnection&) = delete;
     MySQLConnection& operator=(const MySQLConnection&&) = delete;
 
+    static std::shared_ptr<MySQLConnection> create();
+
     operator MYSQL*() { return db; }
 
     // See http://dev.mysql.com/doc/connector-j/en/connector-j-reference-configuration-properties.html
