@@ -1,13 +1,14 @@
-#include "sql/postgresql.h"
+#include "postgresql.h"
 #include "dballe/types.h"
 #include <cstdarg>
 #include <cstdio>
 #include <cstdlib>
-#include "sql/querybuf.h"
+#include "querybuf.h"
 #include <cstdlib>
 #include <cstring>
 #include <arpa/inet.h>
 #include <endian.h>
+#include <unistd.h>
 
 using namespace std;
 using namespace wreport;
@@ -140,7 +141,6 @@ error_postgresql::error_postgresql(PGconn* db, const std::string& msg)
     this->msg = msg;
     this->msg += ": ";
     this->msg += PQerrorMessage(db);
-    fprintf(stderr, "ERROR %s\n", this->msg.c_str());
 }
 
 error_postgresql::error_postgresql(PGresult* res, const std::string& msg)
@@ -148,7 +148,6 @@ error_postgresql::error_postgresql(PGresult* res, const std::string& msg)
     this->msg = msg;
     this->msg += ": ";
     this->msg += PQresultErrorMessage(res);
-    fprintf(stderr, "ERROR %s\n", this->msg.c_str());
 }
 
 error_postgresql::error_postgresql(const std::string& dbmsg, const std::string& msg)
@@ -156,7 +155,6 @@ error_postgresql::error_postgresql(const std::string& dbmsg, const std::string&
     this->msg = msg;
     this->msg += ": ";
     this->msg += dbmsg;
-    fprintf(stderr, "ERROR %s\n", this->msg.c_str());
 }
 
 void error_postgresql::throwf(PGconn* db, const char* fmt, ...)
@@ -194,12 +192,47 @@ PostgreSQLConnection::~PostgreSQLConnection()
     if (db) PQfinish(db);
 }
 
+std::shared_ptr<PostgreSQLConnection> PostgreSQLConnection::create()
+{
+    auto res = std::shared_ptr<PostgreSQLConnection>(new PostgreSQLConnection);
+    res->register_atfork();
+    return res;
+}
+
+void PostgreSQLConnection::fork_prepare()
+{
+}
+
+void PostgreSQLConnection::fork_parent()
+{
+}
+
+void PostgreSQLConnection::fork_child()
+{
+    if (db)
+    {
+        // Close socket to avoid sending close commands to the server that
+        // would interfere with the parent
+        ::close(PQsocket(db));
+        PQfinish(db);
+        db = nullptr;
+        forked = true;
+    }
+}
+
+void PostgreSQLConnection::check_connection()
+{
+    if (forked)
+        throw error_postgresql("server connection closed", "database connections cannot be used after forking");
+}
+
 void PostgreSQLConnection::open_url(const std::string& connection_string)
 {
     url = connection_string;
     db = PQconnectdb(connection_string.c_str());
     if (PQstatus(db) != CONNECTION_OK)
         throw error_postgresql(db, "opening " + connection_string);
+
     init_after_connect();
 }
 
@@ -220,12 +253,16 @@ void PostgreSQLConnection::init_after_connect()
 
 void PostgreSQLConnection::pqexec(const std::string& query)
 {
+    check_connection();
     postgresql::Result res(PQexec(db, query.c_str()));
     res.expect_success(query);
 }
 
 void PostgreSQLConnection::pqexec_nothrow(const std::string& query) noexcept
 {
+    if (forked)
+        return;
+
     postgresql::Result res(PQexec(db, query.c_str()));
     switch (PQresultStatus(res))
     {
@@ -233,7 +270,7 @@ void PostgreSQLConnection::pqexec_nothrow(const std::string& query) noexcept
         case PGRES_TUPLES_OK:
             return;
         default:
-            fprintf(stderr, "cannot execute '%s': %s\n", query.c_str(), PQresultErrorMessage(res));
+            fprintf(stderr, "postgresql cleanup: cannot execute '%s': %s\n", query.c_str(), PQresultErrorMessage(res));
     }
 }
 
@@ -245,7 +282,10 @@ struct PostgreSQLTransaction : public Transaction
     PostgreSQLTransaction(PostgreSQLConnection& conn) : conn(conn)
     {
     }
-    ~PostgreSQLTransaction() { if (!fired) rollback_nothrow(); }
+    ~PostgreSQLTransaction()
+    {
+        if (!fired) rollback_nothrow();
+    }
 
     void commit() override
     {
