@@ -1,6 +1,7 @@
 #include <wreport/utils/tests.h>
 #include <wreport/utils/testrunner.h>
 #include <wreport/utils/term.h>
+#include <wreport/fwd.h>
 #include <dballe/db/tests.h>
 #include <dballe/db/v7/trace.h>
 #include <signal.h>
@@ -34,11 +35,22 @@ int main(int argc,const char* argv[])
     else
         controller.reset(new SimpleTestController(output));
 
-    if (const char* whitelist = getenv("TEST_WHITELIST"))
-        controller->whitelist = whitelist;
+#ifndef WREPORT_VERSION_MAJOR
+#define allowlist whitelist
+#define blocklist blacklist
+#endif
 
-    if (const char* blacklist = getenv("TEST_BLACKLIST"))
-        controller->blacklist = blacklist;
+    if (const char* allowlist = getenv("TEST_WHITELIST"))
+        controller->allowlist = allowlist;
+
+    if (const char* allowlist = getenv("TEST_ONLY"))
+        controller->allowlist = allowlist;
+
+    if (const char* blocklist = getenv("TEST_BLACKLIST"))
+        controller->blocklist = blocklist;
+
+    if (const char* blocklist = getenv("TEST_EXCEPT"))
+        controller->blocklist = blocklist;
 
     auto all_results = tests.run_tests(*controller);
     TestResultStats rstats(all_results);
