@@ -465,8 +465,7 @@ struct __exit__ : MethVarargs<__exit__<Impl>, Impl>
 
         try {
             ReleaseGIL gil;
-            delete self->cur;
-            self->cur = nullptr;
+            self->cur.reset();
         } DBALLE_CATCH_RETURN_PYO
         Py_RETURN_NONE;
     }
@@ -480,7 +479,7 @@ struct DefinitionBase : public Type<Definition, Impl>
 
     static void _dealloc(Impl* self)
     {
-        delete self->cur;
+        self->cur.~shared_ptr();
         Py_TYPE(self)->tp_free(self);
     }
 
@@ -542,7 +541,7 @@ functions. For the keys available, see :ref:`parms_read_station`.
 
 For example::
 
-    with msg.query_stations({...}) as cur:
+    with msg.query_stations(...) as cur:
         for row in cur:
             print("Station:", cur["station"])
 )";
@@ -570,7 +569,7 @@ functions. For the keys available, see :ref:`parms_read_station`.
 
 For example::
 
-    with tr.query_stations({...}) as cur:
+    with tr.query_stations(...) as cur:
         for row in cur:
             print("Station:", cur["station"])
 )";
@@ -597,7 +596,7 @@ functions. For the keys available, see :ref:`parms_read_stationdata`.
 
 For example::
 
-    with msg.query_station_data({...}) as cur:
+    with msg.query_station_data(...) as cur:
         for row in cur:
             print("Station:", cur["station"])
 )";
@@ -624,7 +623,7 @@ functions. For the keys available, see :ref:`parms_read_stationdata`.
 
 For example::
 
-    with tr.query_station_data({...}) as cur:
+    with tr.query_station_data(...) as cur:
         for row in cur:
             print("Station:", cur["station"])
 )";
@@ -651,7 +650,7 @@ functions. For the keys available, see :ref:`parms_read_data`.
 
 For example::
 
-    with msg.query_data({...}) as cur:
+    with msg.query_data(...) as cur:
         for row in cur:
             print("Station:", cur["station"])
 )";
@@ -678,7 +677,7 @@ functions. For the keys available, see :ref:`parms_read_data`.
 
 For example::
 
-    with tr.query_data({...}) as cur:
+    with tr.query_data(...) as cur:
         for row in cur:
             print("Station:", cur["station"])
 )";
@@ -705,7 +704,7 @@ functions. For the keys available, see :ref:`parms_read_summary`.
 
 For example::
 
-    with tr.query_summary({...}) as cur:
+    with tr.query_summary(...) as cur:
         for row in cur:
             print("Station:", cur["station"])
 )";
@@ -732,7 +731,7 @@ functions. For the keys available, see :ref:`parms_read_summary`.
 
 For example::
 
-    with explorer.query_all({...}) as cur:
+    with explorer.query_summary_all(...) as cur:
         for row in cur:
             print("Station:", cur["station"])
 )";
@@ -759,7 +758,7 @@ functions. For the keys available, see :ref:`parms_read_summary`.
 
 For example::
 
-    with explorer.query_all({...}) as cur:
+    with explorer.query_summary_all(...) as cur:
         for row in cur:
             print("Station:", cur["station"])
 )";
@@ -778,12 +777,7 @@ struct message : Getter<message<Impl>, Impl>
     {
         try {
             ensure_valid_iterating_cursor(self);
-            if (self->curmsg)
-            {
-                Py_INCREF(self->curmsg);
-                return self->curmsg;
-            }
-            Py_RETURN_NONE;
+            return (PyObject*)message_create(self->cur->get_message());
         } DBALLE_CATCH_RETURN_PYO
     }
 };
@@ -814,32 +808,6 @@ For example::
 
     GetSetters<remaining<Impl>, message<Impl>, query<Impl>> getsetters;
     Methods<MethGenericEnter<Impl>, __exit__<Impl>, enqi<Impl>, enqd<Impl>, enqs<Impl>, enqf<Impl>> methods;
-
-    static void _dealloc(Impl* self)
-    {
-        delete self->cur;
-        Py_XDECREF(self->curmsg);
-        Py_TYPE(self)->tp_free(self);
-    }
-
-    static PyObject* _iternext(Impl* self)
-    {
-        try {
-            ensure_valid_cursor(self);
-            if (self->cur->next())
-            {
-                std::unique_ptr<Message> msg = self->cur->detach_message();
-                Py_XDECREF(self->curmsg);
-                self->curmsg = nullptr;
-                self->curmsg = (PyObject*)message_create(std::move(msg));
-                Py_INCREF(self);
-                return (PyObject*)self;
-            } else {
-                PyErr_SetNone(PyExc_StopIteration);
-                return nullptr;
-            }
-        } DBALLE_CATCH_RETURN_PYO
-    }
 };
 
 
@@ -859,74 +827,73 @@ DefinitionMessage*           definition_message = nullptr;
 namespace dballe {
 namespace python {
 
-dpy_CursorStation* cursor_create(std::unique_ptr<impl::CursorStation> cur)
+dpy_CursorStation* cursor_create(std::shared_ptr<impl::CursorStation> cur)
 {
     py_unique_ptr<dpy_CursorStation> result(throw_ifnull(PyObject_New(dpy_CursorStation, dpy_CursorStation_Type)));
-    result->cur = cur.release();
+    new (&(result->cur)) std::shared_ptr<CursorStation>(cur);
     return result.release();
 }
 
-dpy_CursorStationDB* cursor_create(std::unique_ptr<db::v7::cursor::Stations> cur)
+dpy_CursorStationDB* cursor_create(std::shared_ptr<db::v7::cursor::Stations> cur)
 {
     py_unique_ptr<dpy_CursorStationDB> result(throw_ifnull(PyObject_New(dpy_CursorStationDB, dpy_CursorStationDB_Type)));
-    result->cur = cur.release();
+    new (&(result->cur)) std::shared_ptr<CursorStation>(cur);
     return result.release();
 }
 
-dpy_CursorStationData* cursor_create(std::unique_ptr<impl::CursorStationData> cur)
+dpy_CursorStationData* cursor_create(std::shared_ptr<impl::CursorStationData> cur)
 {
     py_unique_ptr<dpy_CursorStationData> result(throw_ifnull(PyObject_New(dpy_CursorStationData, dpy_CursorStationData_Type)));
-    result->cur = cur.release();
+    new (&(result->cur)) std::shared_ptr<CursorStationData>(cur);
     return result.release();
 }
 
-dpy_CursorStationDataDB* cursor_create(std::unique_ptr<db::v7::cursor::StationData> cur)
+dpy_CursorStationDataDB* cursor_create(std::shared_ptr<db::v7::cursor::StationData> cur)
 {
     py_unique_ptr<dpy_CursorStationDataDB> result(throw_ifnull(PyObject_New(dpy_CursorStationDataDB, dpy_CursorStationDataDB_Type)));
-    result->cur = cur.release();
+    new (&(result->cur)) std::shared_ptr<CursorStationData>(cur);
     return result.release();
 }
 
-dpy_CursorData* cursor_create(std::unique_ptr<impl::CursorData> cur)
+dpy_CursorData* cursor_create(std::shared_ptr<impl::CursorData> cur)
 {
     py_unique_ptr<dpy_CursorData> result(throw_ifnull(PyObject_New(dpy_CursorData, dpy_CursorData_Type)));
-    result->cur = cur.release();
+    new (&(result->cur)) std::shared_ptr<CursorData>(cur);
     return result.release();
 }
 
-dpy_CursorDataDB* cursor_create(std::unique_ptr<db::v7::cursor::Data> cur)
+dpy_CursorDataDB* cursor_create(std::shared_ptr<db::v7::cursor::Data> cur)
 {
     py_unique_ptr<dpy_CursorDataDB> result(throw_ifnull(PyObject_New(dpy_CursorDataDB, dpy_CursorDataDB_Type)));
-    result->cur = cur.release();
+    new (&(result->cur)) std::shared_ptr<CursorData>(cur);
     return result.release();
 }
 
-dpy_CursorSummaryDB* cursor_create(std::unique_ptr<db::v7::cursor::Summary> cur)
+dpy_CursorSummaryDB* cursor_create(std::shared_ptr<db::v7::cursor::Summary> cur)
 {
     py_unique_ptr<dpy_CursorSummaryDB> result(throw_ifnull(PyObject_New(dpy_CursorSummaryDB, dpy_CursorSummaryDB_Type)));
-    result->cur = cur.release();
+    new (&(result->cur)) std::shared_ptr<CursorSummary>(cur);
     return result.release();
 }
 
-dpy_CursorSummarySummary* cursor_create(std::unique_ptr<db::summary::Cursor<Station>> cur)
+dpy_CursorSummarySummary* cursor_create(std::shared_ptr<db::summary::Cursor<Station>> cur)
 {
     py_unique_ptr<dpy_CursorSummarySummary> result(throw_ifnull(PyObject_New(dpy_CursorSummarySummary, dpy_CursorSummarySummary_Type)));
-    result->cur = cur.release();
+    new (&(result->cur)) std::shared_ptr<CursorSummary>(cur);
     return result.release();
 }
 
-dpy_CursorSummaryDBSummary* cursor_create(std::unique_ptr<db::summary::Cursor<DBStation>> cur)
+dpy_CursorSummaryDBSummary* cursor_create(std::shared_ptr<db::summary::Cursor<DBStation>> cur)
 {
     py_unique_ptr<dpy_CursorSummaryDBSummary> result(throw_ifnull(PyObject_New(dpy_CursorSummaryDBSummary, dpy_CursorSummaryDBSummary_Type)));
-    result->cur = cur.release();
+    new (&(result->cur)) std::shared_ptr<CursorSummary>(cur);
     return result.release();
 }
 
-dpy_CursorMessage* cursor_create(std::unique_ptr<dballe::CursorMessage> cur)
+dpy_CursorMessage* cursor_create(std::shared_ptr<dballe::CursorMessage> cur)
 {
     py_unique_ptr<dpy_CursorMessage> result(throw_ifnull(PyObject_New(dpy_CursorMessage, dpy_CursorMessage_Type)));
-    result->cur = impl::CursorMessage::downcast(std::move(cur)).release();
-    result->curmsg = nullptr;
+    new (&(result->cur)) std::shared_ptr<CursorMessage>(cur);
     return result.release();
 }
 
