@@ -773,7 +773,7 @@ struct export_to_file : MethKwargs<export_to_file<Impl>, Impl>
                 while (cursor->next())
                 {
                     impl::Messages msgs;
-                    msgs.emplace_back(cursor->detach_message());
+                    msgs.emplace_back(cursor->get_message());
                     out->write(exporter->to_binary(msgs));
                 }
                 gil.lock();
@@ -789,7 +789,7 @@ struct export_to_file : MethKwargs<export_to_file<Impl>, Impl>
                 while (cursor->next())
                 {
                     impl::Messages msgs;
-                    msgs.emplace_back(cursor->detach_message());
+                    msgs.emplace_back(cursor->get_message());
                     std::string encoded = exporter->to_binary(msgs);
                     res = pyo_unique_ptr(PyObject_CallMethod(file, (char*)"write", (char*)"y#", encoded.data(), (int)encoded.size()));
                     if (!res)
@@ -1144,29 +1144,7 @@ by automatically creating a temporary transaction and running the equivalent
 :class:`dballe.DB` objects are not constructed explicitly, but via one of the
 :func:`DB.connect` or :func:`DB.connect_test` class methods.
 
-Examples:
-
-::
-
-    # Connect to a database and run a query
-    db = dballe.DB.connect_from_file("db.sqlite")
-    query = {"latmin": 44.0, "latmax": 45.0, "lonmin": 11.0, "lonmax": 12.0}
-
-    # The result is a dballe.Cursor (dballe.CursorData in this case), which can
-    # be iterated to get results as dict objects.
-    with db.transaction() as tr:
-        for row in tr.query_data(query):
-            print(row["lat"], row["lon"], row["var"], row["variable"].format("undefined"))
-
-    # Insert 2 new variables in the database
-    db.insert_data({
-        "lat": 44.5, "lon": 11.4,
-        "level": dballe.Level(1),
-        "trange": dballe.Trange(254),
-        "date": datetime.datetime(2013, 4, 25, 12, 0, 0),
-        "B11101": 22.4,
-        "B12103": 17.2,
-    })
+See :ref:`python_how_dumpdb` and :ref:`python_how_insert` examples.
 )";
 
     GetSetters<> getsetters;
@@ -1276,21 +1254,15 @@ are not visible to other database connections until the transaction is
 committed. If a transaction is rolled back, all changes done with it are
 undone.
 
-Transactions can also be used as context managers, like this:
-
-::
-
-    with db.transaction() as t:
-        for i in range(10):
-            t.insert({
-                "lat": 44.5 + i, "lon": 11.4 + i, "level": (1,),
-                "trange": (254,), "date": datetime.datetime(2013, 4, 25, 12, 0, 0),
-                "B11101": 22.4 + i, "B12103": 17.2
-            })
+Transactions can also be used as context managers, which calls
+:py:meth:`dballe.Transaction.commit` automatically, or
+:py:meth:`dballe.Transaction.rollback` if the code raised an exception:
 
 The dballe.Transaction methods are the same as those in dballe.DB. The version
 in dballe.DB is implemented by automatically creating a temporary transaction
 and running the dballe.Transaction method inside it.
+
+See :ref:`python_how_dumpdb` and :ref:`python_how_insert` examples.
 )";
 
     GetSetters<> getsetters;
