@@ -61,15 +61,6 @@ PyTypeObject* dpy_DBExplorerUpdate_Type = nullptr;
 
 }
 
-namespace dballe {
-namespace python {
-
-template<typename Station>
-inline PyObject* to_python(const db::summary::StationEntry<Station>& s) { return to_python(s.station); }
-
-}
-}
-
 namespace {
 
 struct All {};
@@ -104,32 +95,47 @@ template<> const db::BaseSummary<DBStation>& get_summary<DBStation, Selected>(dp
 
 template<typename Station>
 struct get_stations {
-    static const db::summary::StationEntries<Station>& get(const db::BaseSummary<Station>& summary) { return summary.stations(); }
+    static bool iter(const db::BaseSummary<Station>& summary, std::function<bool(PyObject*)> dest)
+    {
+        return summary.stations([&](const Station& s) { return dest(to_python(s)); });
+    }
 };
 
 template<typename Station>
 struct get_reports {
-    static const core::SortedSmallUniqueValueSet<std::string>& get(const db::BaseSummary<Station>& summary) { return summary.reports(); }
+    static bool iter(const db::BaseSummary<Station>& summary, std::function<bool(PyObject*)> dest)
+    {
+        return summary.reports([&](const std::string& r) { return dest(to_python(r)); });
+    }
 };
 
 template<typename Station>
 struct get_levels {
-    static const core::SortedSmallUniqueValueSet<dballe::Level>& get(const db::BaseSummary<Station>& summary) { return summary.levels(); }
+    static bool iter(const db::BaseSummary<Station>& summary, std::function<bool(PyObject*)> dest)
+    {
+        return summary.levels([&](const Level& lev) { return dest(to_python(lev)); });
+    }
 };
 
 template<typename Station>
 struct get_tranges {
-    static const core::SortedSmallUniqueValueSet<dballe::Trange>& get(const db::BaseSummary<Station>& summary) { return summary.tranges(); }
+    static bool iter(const db::BaseSummary<Station>& summary, std::function<bool(PyObject*)> dest)
+    {
+        return summary.tranges([&](const Trange& tr) { return dest(to_python(tr)); });
+    }
 };
 
 template<typename Station>
 struct get_varcodes {
-    static const core::SortedSmallUniqueValueSet<wreport::Varcode>& get(const db::BaseSummary<Station>& summary) { return summary.varcodes(); }
+    static bool iter(const db::BaseSummary<Station>& summary, std::function<bool(PyObject*)> dest)
+    {
+        return summary.varcodes([&](const wreport::Varcode& v) { return dest(to_python(v)); });
+    }
 };
 
 namespace explorer {
 
-template<typename Base, typename Station, typename Scope, typename GET>
+template<typename Base, typename Station, typename Scope, typename ITER>
 struct BaseGetter : public Getter<Base, typename ImplTraits<Station>::Impl>
 {
     typedef typename ImplTraits<Station>::Impl Impl;
@@ -137,17 +143,14 @@ struct BaseGetter : public Getter<Base, typename ImplTraits<Station>::Impl>
     {
         try {
             const auto& summary = get_summary<Station, Scope>(*self);
-            const auto& stations = GET::get(summary);
-            pyo_unique_ptr result(PyList_New(stations.size()));
+            pyo_unique_ptr result(PyList_New(0));
 
-            unsigned idx = 0;
-            for (const auto& entry: stations)
-            {
-                pyo_unique_ptr station(to_python(entry));
-                if (PyList_SetItem(result, idx, station.release()))
-                    return nullptr;
-                ++idx;
-            }
+            ITER::iter(summary, [&](PyObject* o) {
+                pyo_unique_ptr val(o);
+                if (PyList_Append(result, val.release()) == -1)
+                    throw PythonException();
+                return true;
+            });
 
             return result.release();
         } DBALLE_CATCH_RETURN_PYO
@@ -388,7 +391,7 @@ struct BaseQuerySummary : public MethKwargs<Base, typename ImplTraits<Station>::
             const auto& summary = get_summary<Station, Scope>(*self);
             auto res = summary.query_summary(*query);
             gil.lock();
-            return (PyObject*)cursor_create(std::unique_ptr<db::summary::Cursor<Station>>(dynamic_cast<db::summary::Cursor<Station>*>(res.release())));
+            return (PyObject*)cursor_create(std::static_pointer_cast<db::summary::Cursor<Station>>(res));
         } DBALLE_CATCH_RETURN_PYO
     }
 };
@@ -404,6 +407,9 @@ struct query_summary_doc_traits<Station>
     constexpr static const char* returns = "dballe.CursorSummarySummary";
     constexpr static const char* doc = R"(
 :return: a cursor to iterate the query results (see :py:class:`dballe.CursorSummarySummary`)
+
+Adding ``query=details`` will be ignored for Explorers and considered always
+present, as details are always provided in results.
 )";
 };
 
@@ -413,6 +419,9 @@ struct query_summary_doc_traits<DBStation>
     constexpr static const char* returns = "dballe.CursorSummaryDBSummary";
     constexpr static const char* doc = R"(
 :return: a cursor to iterate the query results (see :py:class:`dballe.CursorSummaryDBSummary`)
+
+Adding ``query=details`` will be ignored for Explorers and considered always
+present, as details are always provided in results.
 )";
 };
 
@@ -435,6 +444,30 @@ struct query_summary : public BaseQuerySummary<query_summary<Station>, Station,
     constexpr static const char* doc = query_summary_doc_traits<Station>::doc;
 };
 
+template<typename Station>
+struct __exit__ : public MethVarargs<__exit__<Station>, typename ImplTraits<Station>::Impl>
+{
+    typedef typename ImplTraits<Station>::Impl Impl;
+    constexpr static const char* name = "__exit__";
+    constexpr static const char* doc = "Context manager __exit__";
+    static PyObject* run(Impl* self, PyObject* args)
+    {
+        PyObject* exc_type;
+        PyObject* exc_val;
+        PyObject* exc_tb;
+        if (!PyArg_ParseTuple(args, "OOO", &exc_type, &exc_val, &exc_tb))
+            return nullptr;
+
+        try {
+            ReleaseGIL gil;
+            delete self->explorer;
+            self->explorer = nullptr;
+        } DBALLE_CATCH_RETURN_PYO
+
+        Py_RETURN_NONE;
+    }
+};
+
 
 template<class Station>
 struct Definition : Type<Definition<Station>, typename ImplTraits<Station>::Impl>
@@ -442,7 +475,26 @@ struct Definition : Type<Definition<Station>, typename ImplTraits<Station>::Impl
     typedef typename ImplTraits<Station>::Impl Impl;
     static const char* name;
     static const char* qual_name;
-    constexpr static const char* doc = "Browser for a summary of DB-All-e database of message contents";
+    constexpr static const char* doc = R"(
+Browser for a summary of DB-All-e database of message contents.
+
+Use :meth:`rebuild` or :meth:`update` to populate the explorer.
+
+If a file name is passed to the constructor, the Explorer automatically loads
+contents from the file (if it exists), and saves them to the file on update.
+
+The persistence file is in JSON format if the file name ends with ``.json`` or
+if no Xapian support is compiled in. Otherwise, the Explorer will persist using
+an indexed Xapian database.
+
+::
+
+    # Populate an explorer with the contents of a database
+    explorer = dballe.Explorer()
+    with explorer.rebuild() as update:
+        with db.transaction() as tr:
+            update.add_db(tr)
+)";
     GetSetters<
         GetAllStations<Station>, GetStations<Station>,
         GetAllReports<Station>, GetReports<Station>,
@@ -450,7 +502,10 @@ struct Definition : Type<Definition<Station>, typename ImplTraits<Station>::Impl
         GetAllTranges<Station>, GetTranges<Station>,
         GetAllVarcodes<Station>, GetVarcodes<Station>,
         GetAllStats<Station>, GetStats<Station>> getsetters;
-    Methods<set_filter<Station>, rebuild<Station>, update<Station>, to_json<Station>, query_summary_all<Station>, query_summary<Station>> methods;
+    Methods<MethGenericEnter<typename ImplTraits<Station>::Impl>,
+           __exit__<Station>, set_filter<Station>, rebuild<Station>,
+           update<Station>, to_json<Station>, query_summary_all<Station>,
+           query_summary<Station>> methods;
 
     static void _dealloc(Impl* self)
     {
@@ -472,12 +527,16 @@ struct Definition : Type<Definition<Station>, typename ImplTraits<Station>::Impl
 
     static int _init(Impl* self, PyObject* args, PyObject* kw)
     {
-        static const char* kwlist[] = { nullptr };
-        if (!PyArg_ParseTupleAndKeywords(args, kw, "", const_cast<char**>(kwlist)))
+        static const char* kwlist[] = { "pathname", nullptr };
+        const char* pathname = nullptr;
+        if (!PyArg_ParseTupleAndKeywords(args, kw, "|s", const_cast<char**>(kwlist), &pathname))
             return -1;
 
         try {
-            self->explorer = new typename ImplTraits<Station>::cpp_impl;
+            if (pathname)
+                self->explorer = new typename ImplTraits<Station>::cpp_impl(pathname);
+            else
+                self->explorer = new typename ImplTraits<Station>::cpp_impl;
         } DBALLE_CATCH_RETURN_INT
 
         return 0;
@@ -577,7 +636,7 @@ struct add_explorer : public MethKwargs<add_explorer<Station>, typename ImplTrai
     typedef typename ImplTraits<Station>::UpdateImpl Impl;
     constexpr static const char* name = "add_explorer";
     constexpr static const char* doc = R"(
-Add the contents of the given Explorer or DBExplorer to the Explorer.
+Add the currently selected contents of the given Explorer or DBExplorer to the Explorer.
 )";
     static PyObject* run(Impl* self, PyObject* args, PyObject* kw)
     {
@@ -614,7 +673,7 @@ struct add_messages : public MethKwargs<add_messages<Station>, typename ImplTrai
     constexpr static const char* doc = R"(
 Add dballe.Message objects to the explorer.
 
-It takes the same messages argument of dballe.DB.import_messages
+It takes the same messages argument of :func:`dballe.DB.import_messages`.
 )";
     [[noreturn]] static void throw_typeerror()
     {
@@ -624,9 +683,11 @@ It takes the same messages argument of dballe.DB.import_messages
 
     static PyObject* run(Impl* self, PyObject* args, PyObject* kw)
     {
-        static const char* kwlist[] = { "messages", nullptr };
+        static const char* kwlist[] = { "messages", "station_data", "data", nullptr };
         PyObject* obj;
-        if (!PyArg_ParseTupleAndKeywords(args, kw, "O", const_cast<char**>(kwlist), &obj))
+        int station_data = 1;
+        int data = 1;
+        if (!PyArg_ParseTupleAndKeywords(args, kw, "O|pp", const_cast<char**>(kwlist), &obj, &station_data, &data))
             return nullptr;
 
         try {
@@ -642,7 +703,7 @@ It takes the same messages argument of dballe.DB.import_messages
                 while (auto binmsg = impf->file->file->file().read())
                 {
                     auto messages = impf->importer->importer->from_binary(binmsg);
-                    self->update.add_messages(messages);
+                    self->update.add_messages(messages, station_data, data);
                 }
                 Py_RETURN_NONE;
             }
@@ -658,7 +719,7 @@ It takes the same messages argument of dballe.DB.import_messages
                 {
                     pyo_unique_ptr o(throw_ifnull(PySequence_ITEM(obj, i)));
                     if (!dpy_Message_Check(o)) throw_typeerror();
-                    self->update.add_message(*(((dpy_Message*)o.get())->message));
+                    self->update.add_message(*(((dpy_Message*)o.get())->message), station_data, data);
                 }
                 Py_RETURN_NONE;
             }
@@ -670,7 +731,7 @@ It takes the same messages argument of dballe.DB.import_messages
                 while (pyo_unique_ptr item = PyIter_Next(iterator))
                 {
                     if (!dpy_Message_Check(item)) throw_typeerror();
-                    self->update.add_message(*(((dpy_Message*)item.get())->message));
+                    self->update.add_message(*(((dpy_Message*)item.get())->message), station_data, data);
                 }
                 if (PyErr_Occurred()) return nullptr;
                 Py_RETURN_NONE;
@@ -733,6 +794,12 @@ dpy_Explorer* explorer_create()
     return explorer_create(move(explorer));
 }
 
+dpy_Explorer* explorer_create(const std::string& pathname)
+{
+    unique_ptr<db::Explorer> explorer(new db::Explorer(pathname));
+    return explorer_create(move(explorer));
+}
+
 dpy_Explorer* explorer_create(std::unique_ptr<db::Explorer> explorer)
 {
     dpy_Explorer* result = PyObject_New(dpy_Explorer, dpy_Explorer_Type);
@@ -748,6 +815,12 @@ dpy_DBExplorer* dbexplorer_create()
     return dbexplorer_create(move(explorer));
 }
 
+dpy_DBExplorer* dbexplorer_create(const std::string& pathname)
+{
+    unique_ptr<db::DBExplorer> explorer(new db::DBExplorer(pathname));
+    return dbexplorer_create(move(explorer));
+}
+
 dpy_DBExplorer* dbexplorer_create(std::unique_ptr<db::DBExplorer> explorer)
 {
     dpy_DBExplorer* result = PyObject_New(dpy_DBExplorer, dpy_DBExplorer_Type);
