@@ -92,6 +92,18 @@ struct l2 : Getter<l2, dpy_Level>
     }
 };
 
+struct describe : MethNoargs<describe, dpy_Level>
+{
+    constexpr static const char* name = "describe";
+    constexpr static const char* doc = "Return a string description for this level";
+    static PyObject* run(Impl* self)
+    {
+        try {
+            return to_python(self->val.describe());
+        } DBALLE_CATCH_RETURN_PYO
+    }
+};
+
 struct Definition : public Type<Definition, dpy_Level>
 {
     constexpr static const char* name = "Level";
@@ -103,7 +115,7 @@ Constructor: Level(ltype1: int=None, l1: int=None, ltype2: int=None, l2: int=Non
 )";
 
     GetSetters<ltype1, l1, ltype2, l2> getsetters;
-    Methods<> methods;
+    Methods<describe> methods;
 
     static PyObject* _str(Impl* self)
     {
@@ -126,10 +138,10 @@ Constructor: Level(ltype1: int=None, l1: int=None, ltype2: int=None, l2: int=Non
     {
         try {
             py_unique_ptr<PyTupleObject> res((PyTupleObject*)throw_ifnull(PyTuple_New(4)));
-            PyTuple_SET_ITEM(res, 0, dballe_int_to_python(self->val.ltype1));
-            PyTuple_SET_ITEM(res, 1, dballe_int_to_python(self->val.l1));
-            PyTuple_SET_ITEM(res, 2, dballe_int_to_python(self->val.ltype2));
-            PyTuple_SET_ITEM(res, 3, dballe_int_to_python(self->val.l2));
+            PyTuple_SET_ITEM(res.get(), 0, dballe_int_to_python(self->val.ltype1));
+            PyTuple_SET_ITEM(res.get(), 1, dballe_int_to_python(self->val.l1));
+            PyTuple_SET_ITEM(res.get(), 2, dballe_int_to_python(self->val.ltype2));
+            PyTuple_SET_ITEM(res.get(), 3, dballe_int_to_python(self->val.l2));
             return PyObject_GetIter((PyObject*)res.get());
         } DBALLE_CATCH_RETURN_PYO
     }
@@ -224,6 +236,18 @@ struct p2 : Getter<p2, dpy_Trange>
     }
 };
 
+struct describe : MethNoargs<describe, dpy_Trange>
+{
+    constexpr static const char* name = "describe";
+    constexpr static const char* doc = "Return a string description for this time range";
+    static PyObject* run(Impl* self)
+    {
+        try {
+            return to_python(self->val.describe());
+        } DBALLE_CATCH_RETURN_PYO
+    }
+};
+
 struct Definition : public Type<Definition, dpy_Trange>
 {
     constexpr static const char* name = "Trange";
@@ -235,7 +259,7 @@ Constructor: Trange(pind: int=None, p1: int=None, p2: int=None)
 )";
 
     GetSetters<pind, p1, p2> getsetters;
-    Methods<> methods;
+    Methods<describe> methods;
 
     static PyObject* _str(Impl* self)
     {
@@ -258,9 +282,9 @@ Constructor: Trange(pind: int=None, p1: int=None, p2: int=None)
     {
         try {
             py_unique_ptr<PyTupleObject> res((PyTupleObject*)throw_ifnull(PyTuple_New(3)));
-            PyTuple_SET_ITEM(res, 0, dballe_int_to_python(self->val.pind));
-            PyTuple_SET_ITEM(res, 1, dballe_int_to_python(self->val.p1));
-            PyTuple_SET_ITEM(res, 2, dballe_int_to_python(self->val.p2));
+            PyTuple_SET_ITEM(res.get(), 0, dballe_int_to_python(self->val.pind));
+            PyTuple_SET_ITEM(res.get(), 1, dballe_int_to_python(self->val.p1));
+            PyTuple_SET_ITEM(res.get(), 2, dballe_int_to_python(self->val.p2));
             return PyObject_GetIter((PyObject*)res.get());
         } DBALLE_CATCH_RETURN_PYO
     }
@@ -394,6 +418,40 @@ struct lon : Getter<lon<Station>, typename StationImplTraits<Station>::Impl>
     }
 };
 
+template<typename Station>
+struct ilat : Getter<ilat<Station>, typename StationImplTraits<Station>::Impl>
+{
+    typedef typename StationImplTraits<Station>::Impl Impl;
+    constexpr static const char* name = "ilat";
+    constexpr static const char* doc = "station latitude (as integer)";
+    static PyObject* get(Impl* self, void* closure)
+    {
+        try {
+            if (self->val.coords.lat == MISSING_INT)
+                Py_RETURN_NONE;
+            else
+                return PyLong_FromLong(self->val.coords.lat);
+        } DBALLE_CATCH_RETURN_PYO
+    }
+};
+
+template<typename Station>
+struct ilon : Getter<ilon<Station>, typename StationImplTraits<Station>::Impl>
+{
+    typedef typename StationImplTraits<Station>::Impl Impl;
+    constexpr static const char* name = "ilon";
+    constexpr static const char* doc = "station longitude (as integer)";
+    static PyObject* get(Impl* self, void* closure)
+    {
+        try {
+            if (self->val.coords.lon == MISSING_INT)
+                Py_RETURN_NONE;
+            else
+                return PyLong_FromLong(self->val.coords.lon);
+        } DBALLE_CATCH_RETURN_PYO
+    }
+};
+
 template<typename Station>
 struct ident : Getter<ident<Station>, typename StationImplTraits<Station>::Impl>
 {
@@ -476,7 +534,7 @@ Station information.
 
 Constructor: Station(report: str, lat: float, lon: float, ident: str=None)
 )";
-    GetSetters<report<Station>, lat<Station>, lon<Station>, ident<Station>> getsetters;
+    GetSetters<report<Station>, lat<Station>, lon<Station>, ilat<Station>, ilon<Station>, ident<Station>> getsetters;
     static Station from_args(PyObject* args, PyObject* kw)
     {
         static const char* kwlist[] = { "report", "lat", "lon", "ident", nullptr };
@@ -505,7 +563,7 @@ Station information with database ID.
 
 Constructor: Station(report: str, id: int, lat: float, lon: float, ident: str=None)
 )";
-    GetSetters<report<Station>, id, lat<Station>, lon<Station>, ident<Station>> getsetters;
+    GetSetters<report<Station>, id, lat<Station>, lon<Station>, ilat<Station>, ilon<Station>, ident<Station>> getsetters;
     static DBStation from_args(PyObject* args, PyObject* kw)
     {
         static const char* kwlist[] = { "report", "id", "lat", "lon", "ident", nullptr };
@@ -548,24 +606,41 @@ PyObject* datetime_to_python(const Datetime& dt)
             dt.hour, dt.minute, dt.second, 0));
 }
 
+static int get_attr_int(PyObject* o, const char* name)
+{
+    pyo_unique_ptr res(throw_ifnull(PyObject_GetAttrString(o, name)));
+    return from_python<int>(res);
+}
+
 Datetime datetime_from_python(PyObject* dt)
 {
     if (dt == NULL || dt == Py_None)
         return Datetime();
 
-    if (!PyDateTime_Check(dt))
+    if (PyDateTime_Check(dt))
     {
-        PyErr_SetString(PyExc_TypeError, "value must be an instance of datetime.datetime");
-        throw PythonException();
+        return Datetime(
+            PyDateTime_GET_YEAR((PyDateTime_DateTime*)dt),
+            PyDateTime_GET_MONTH((PyDateTime_DateTime*)dt),
+            PyDateTime_GET_DAY((PyDateTime_DateTime*)dt),
+            PyDateTime_DATE_GET_HOUR((PyDateTime_DateTime*)dt),
+            PyDateTime_DATE_GET_MINUTE((PyDateTime_DateTime*)dt),
+            PyDateTime_DATE_GET_SECOND((PyDateTime_DateTime*)dt));
     }
 
+    // Fall back to duck typing, to catch creative cases such as
+    // cftime.datetime instances. See https://unidata.github.io/cftime/api.html
     return Datetime(
-        PyDateTime_GET_YEAR((PyDateTime_DateTime*)dt),
-        PyDateTime_GET_MONTH((PyDateTime_DateTime*)dt),
-        PyDateTime_GET_DAY((PyDateTime_DateTime*)dt),
-        PyDateTime_DATE_GET_HOUR((PyDateTime_DateTime*)dt),
-        PyDateTime_DATE_GET_MINUTE((PyDateTime_DateTime*)dt),
-        PyDateTime_DATE_GET_SECOND((PyDateTime_DateTime*)dt));
+        get_attr_int(dt, "year"),
+        get_attr_int(dt, "month"),
+        get_attr_int(dt, "day"),
+        get_attr_int(dt, "hour"),
+        get_attr_int(dt, "minute"),
+        get_attr_int(dt, "second")
+    );
+
+    // PyErr_SetString(PyExc_TypeError, "value must be an instance of datetime.datetime");
+    // throw PythonException();
 }
 
 DatetimeRange datetimerange_from_python(PyObject* val)
@@ -806,7 +881,7 @@ std::unique_ptr<Query> query_from_python(PyObject* o)
 {
     core::Query* q;
     std::unique_ptr<dballe::Query> res(q = new core::Query);
-    if (!o || o == Py_None)
+    if (!o || o == Py_None || o == Py_Ellipsis)
         return res;
 
     if (PyDict_Check(o))
@@ -900,7 +975,7 @@ DataPtr::DataPtr(PyObject* from_python)
         dpy_CursorStationDataDB* cur = (dpy_CursorStationDataDB*)from_python;
         data->station = cur->cur->get_station();
         data->station.id = MISSING_INT;
-        data->values.set(*cur->cur->rows->value.get());
+        data->values.set(*cur->cur->row().value.get());
         return;
     }
 
@@ -913,7 +988,7 @@ DataPtr::DataPtr(PyObject* from_python)
         data->datetime = cur->cur->get_datetime();
         data->level = cur->cur->get_level();
         data->trange = cur->cur->get_trange();
-        data->values.set(*cur->cur->rows->value.get());
+        data->values.set(*cur->cur->row().value.get());
         return;
     }
 
@@ -982,9 +1057,10 @@ int dballe_int_lat_from_python(PyObject* o)
     {
         PyErr_Clear();
     } else {
-        pyo_unique_ptr six(throw_ifnull(PyLong_FromLong(5)));
-        pyo_unique_ptr scaled(throw_ifnull(PyObject_CallFunctionObjArgs(scaleb, six.get(), nullptr)));
-        int res = PyLong_AsLong(scaled);
+        pyo_unique_ptr digits(throw_ifnull(PyLong_FromLong(5)));
+        pyo_unique_ptr scaled(throw_ifnull(PyObject_CallFunctionObjArgs(scaleb, digits.get(), nullptr)));
+        pyo_unique_ptr as_int(throw_ifnull(PyObject_CallMethod(scaled, "__int__", nullptr)));
+        int res = PyLong_AsLong(as_int);
         if (res == -1 && PyErr_Occurred())
             throw PythonException();
         return res;
@@ -1022,9 +1098,10 @@ int dballe_int_lon_from_python(PyObject* o)
     {
         PyErr_Clear();
     } else {
-        pyo_unique_ptr six(throw_ifnull(PyLong_FromLong(5)));
-        pyo_unique_ptr scaled(throw_ifnull(PyObject_CallFunctionObjArgs(scaleb, six.get(), nullptr)));
-        int res = PyLong_AsLong(scaled);
+        pyo_unique_ptr digits(throw_ifnull(PyLong_FromLong(5)));
+        pyo_unique_ptr scaled(throw_ifnull(PyObject_CallFunctionObjArgs(scaleb, digits.get(), nullptr)));
+        pyo_unique_ptr as_int(throw_ifnull(PyObject_CallMethod(scaled, "__int__", nullptr)));
+        int res = PyLong_AsLong(as_int);
         if (res == -1 && PyErr_Occurred())
             throw PythonException();
         return res;
