@@ -186,6 +186,8 @@ static void print_item_header(const Item& item)
                 if (item.bulletin != NULL)
                     print_crex_header(*dynamic_cast<const CrexBulletin*>(item.bulletin));
                 break;
+            case Encoding::JSON:
+                break;
         }
     } else if (item.msgs) {
         printf(" message: %zd subsets:", item.msgs->size());
@@ -235,6 +237,8 @@ struct Head : public cmdline::Action
                 if (item.bulletin == NULL) return true;
                 dump_crex_header(*item.rmsg, *dynamic_cast<const CrexBulletin*>(item.bulletin)); puts(".");
                 break;
+            case Encoding::JSON:
+                throw error_unimplemented("Head json");
         }
         return true;
     }
@@ -383,6 +387,9 @@ struct DumpMessage : public cmdline::Action
                     print_subsets(*item.bulletin);
                     break;
                 }
+            case Encoding::JSON:
+                printf("contents only available with --interpreted\n");
+                break;
         }
         return true;
     }
@@ -879,7 +886,7 @@ struct Convert : public cmdline::Subcommand
         opts.push_back({ "type", 't', POPT_ARG_STRING, &readeropts.input_type, 0,
             "format of the input data ('bufr', 'crex', 'json', 'csv')", "type" });
         opts.push_back({ "dest", 'd', POPT_ARG_STRING, &op_output_type, 0,
-            "format of the data in output ('bufr', 'crex')", "type" });
+            "format of the data in output ('bufr', 'crex', 'json')", "type" });
         opts.push_back({ "rejected", 0, POPT_ARG_STRING, &readeropts.fail_file_name, 0,
             "write unprocessed data to this file", "fname" });
         opts.push_back({ "template", 0, POPT_ARG_STRING, &op_output_template, 0,
@@ -943,7 +950,7 @@ struct Convert : public cmdline::Subcommand
                 conv.file = File::create(string_to_encoding(op_output_type), op_output_file, "w").release();
         }
 
-        conv.exporter = Exporter::create(conv.file->encoding(), opts).release();
+        conv.set_exporter(conv.file->encoding(), opts);
 
         reader.read(get_filenames(optCon), conv);
 
