@@ -51,8 +51,8 @@ struct VarinfoPrinter : public cmdline::Subcommand
     void add_to_optable(std::vector<poptOption>& opts) const override
     {
         Subcommand::add_to_optable(opts);
-        opts.push_back(poptOption{ "csv", 'c', POPT_ARG_NONE, (void*)&op_csv, 0, "output variables in CSV format" });
-        opts.push_back(poptOption{ "crex", 0,  POPT_ARG_NONE, (void*)&op_crex, 0, "read CREX entries instead of BUFR" });
+        opts.push_back(poptOption{ "csv", 'c', POPT_ARG_NONE, (void*)&op_csv, 0, "output variables in CSV format", 0 });
+        opts.push_back(poptOption{ "crex", 0,  POPT_ARG_NONE, (void*)&op_crex, 0, "read CREX entries instead of BUFR", 0 });
     }
 
     const Vartable* load_vartable(const char* id)
@@ -212,7 +212,7 @@ struct Grep : public VarinfoPrinter
 #if HAVE_STRCASESTR
             if (strcasestr(info->desc, pattern) != NULL)
 #else
-#warning dbatbl grep is case sensite on this sytstem, since strcasestr is not available
+#warning dbatbl grep is case sensitive on this sytstem, since strcasestr is not available
             if (strstr(info->desc, pattern) != NULL)
 #endif
                 print_varinfo(info);
